require "cgi"
require "erb"
require "stringio"
require "nkf"


module CGIPPModule
    
    def initialize_cgipp()
      @header_done= false
      @in_cookie= CGI::Cookie.parse(self.raw_cookie).freeze()
      @out_cookie= []
    end
    
    def [](name, n= 0)
      return self.params[name][n] || ""
    end
    
    def param_str(name, default= nil, n= 0)
      return self[name, n]!="" ? self[name, n] : default
    end
    
    def param_int(name, default= nil, n= 0)
      return self[name, n]!="" ? self[name, n].to_i() : default
    end
    
    def param_float(name, default= nil, n= 0)
      return self[name, n]!="" ? self[name, n].to_f() : default
    end
    
    def param_bool(name, n= 0)
      return self[name, n]!=""
    end
    
    def cookie
      return @in_cookie
    end
    
    def set_cookie(*args)
      @out_cookie.push(CGI::Cookie.new(*args))
    end
    
    def run_erb(file_name, binding= TOP_LEVEL_BINDING, options= {})
      print_header(options)
      script= open(file_name){ |f| f.read() }
      ERB.new(script, nil, "<>").run(binding)
    end
    
    def dprintf(*args)
      print_header()
      print("<pre>")
      printf(*args)
      print("</pre>")
    end
    
    def dp(*args)
      print_header()
      print("<pre>")
      p(*args)
      print("</pre>")
    end
    
    def print_header(options= {})
      if !@header_done
        print(header(header_base_options().merge(options)))
      end
      @header_done= true
    end
    
    def unescape_unicode(str)
      return str.gsub(/%u([0-9a-fA-F]{1,4})|./n) do
        $1 ? NKF.nkf("-w80W16B", [$1].pack("H*")) : $&
      end
    end
    
  private
    
    def header_base_options()
      return {"cookie" => @out_cookie}
    end
    
end


class CGIPP < CGI
    
    def initialize(*args)
      super(*args)
      extend(CGIPPModule)
      initialize_cgipp()
    end
    
end


$cgi= CGIPP.new()
